@@ -4,7 +4,7 @@
 //!
 //! ```
 //! # use {evoke::client::{ClientSystem, DummyLocalPlayer}, core::marker::PhantomData};
-//! # async fn client<'a>(world: &'a mut hecs::World, scope: &'a scoped_arena::Scope<'a>) -> eyre::Result<()> {
+//! # async fn client<'a>(world: &'a mut edict::World, scope: &'a scoped_arena::Scope<'a>) -> eyre::Result<()> {
 //! #[derive(serde::Deserialize)]
 //! struct Foo;
 //!
@@ -31,12 +31,17 @@ use std::{
 use alkahest::{Bytes, FixedUsize, Pack};
 use bincode::Options as _;
 use bitsetium::BitTest;
+use edict::{
+    component::Component,
+    entity::EntityId,
+    query::{Fetch, NonTrackingQuery, Query},
+    world::{EntityError, World},
+};
 use evoke_core::{
     channel::tcp::TcpChannel,
     client_server::{ClientSession, PlayerId},
 };
 use eyre::WrapErr as _;
-use hecs::{Component, Entity, Fetch, Query, World};
 use scoped_arena::Scope;
 use tokio::net::{TcpStream, ToSocketAddrs};
 use tracing::instrument;
@@ -69,14 +74,14 @@ pub type DescriptorPackType<T> = <T as Descriptor>::Pack;
 /// [this blanket impl]: trait.Descriptor.html#impl-Descriptor
 pub trait Descriptor: 'static {
     /// Query for components.
-    type Query: Query;
+    type Query: Query + NonTrackingQuery;
 
     /// Data ready to be unpacked.
     type Pack: serde::de::DeserializeOwned;
 
     /// Insert components managed by this descriptor.
     /// This method is called during replication if descriptor's query cannot be satisfied.
-    fn insert(pack: Self::Pack, entity: Entity, world: &mut World);
+    fn insert(pack: Self::Pack, entity: &EntityId, world: &mut World);
 
     /// Modify components managed by this descriptor.
     /// This method is called during replication if descriptor's query is satisfied.
@@ -84,7 +89,7 @@ pub trait Descriptor: 'static {
 
     /// Remove components managed by this descriptor.
     /// This method is called during replication if descriptor's query is satisfied.
-    fn remove(entity: Entity, world: &mut World);
+    fn remove(entity: &EntityId, world: &mut World);
 }
 
 impl<T> Descriptor for T
@@ -98,12 +103,12 @@ where
         *item = pack;
     }
 
-    fn insert(pack: T, entity: Entity, world: &mut World) {
-        world.insert_one(entity, pack).unwrap();
+    fn insert(pack: T, entity: &EntityId, world: &mut World) {
+        world.try_insert(entity, pack).unwrap();
     }
 
-    fn remove(entity: Entity, world: &mut World) {
-        let _ = world.remove_one::<T>(entity);
+    fn remove(entity: &EntityId, world: &mut World) {
+        let _ = world.remove::<T>(entity);
     }
 }
 
@@ -116,11 +121,11 @@ impl Descriptor for PlayerIdDescriptor {
     fn modify(pack: NonZeroU64, item: &mut PlayerId) {
         item.0 = pack;
     }
-    fn insert(pack: NonZeroU64, entity: Entity, world: &mut World) {
-        world.insert_one(entity, PlayerId(pack)).unwrap();
+    fn insert(pack: NonZeroU64, entity: &EntityId, world: &mut World) {
+        world.try_insert(entity, PlayerId(pack)).unwrap();
     }
-    fn remove(entity: Entity, world: &mut World) {
-        world.remove_one::<PlayerId>(entity).unwrap();
+    fn remove(entity: &EntityId, world: &mut World) {
+        world.remove::<PlayerId>(entity).unwrap();
     }
 }
 
@@ -154,7 +159,7 @@ enum TrackReplicate {
 }
 
 impl TrackReplicate {
-    fn from_mask<'a, B>(idx: usize, mask: &B) -> Result<Self, BadMessage>
+    fn from_mask<B>(idx: usize, mask: &B) -> Result<Self, BadMessage>
     where
         B: BitTest,
     {
@@ -169,7 +174,7 @@ impl TrackReplicate {
 
 fn replicate_one<'a, T>(
     track: TrackReplicate,
-    entity: Entity,
+    entity: &EntityId,
     world: &mut World,
     cursor: &'a mut Cursor<&[u8]>,
 ) -> Result<(), BadMessage>
@@ -180,8 +185,8 @@ where
 
     let item = match world.query_one_mut::<T::Query>(entity) {
         Ok(item) => Some(item),
-        Err(hecs::QueryOneError::Unsatisfied) => None,
-        Err(hecs::QueryOneError::NoSuchEntity) => {
+        Err(EntityError::MissingComponents) => None,
+        Err(EntityError::NoSuchEntity) => {
             unreachable!("EntityMapper must guarantee that entity is alive")
         }
     };
@@ -240,7 +245,7 @@ impl Replicator for () {
             // Begin for each component + player_id.
 
             let pid = TrackReplicate::from_mask(0, &header.mask)?;
-            replicate_one::<PlayerIdDescriptor>(pid, entity, world, &mut cursor)?;
+            replicate_one::<PlayerIdDescriptor>(pid, &entity, world, &mut cursor)?;
 
             // End for each component + player_id.
         }
@@ -252,7 +257,7 @@ impl Replicator for () {
             })?;
 
             if let Some(entity) = mapper.get(nid) {
-                let _ = world.despawn(entity);
+                let _ = world.despawn(&entity);
             }
         }
 
@@ -277,7 +282,7 @@ pub trait LocalPlayerPack<'a> {
 /// Typical implementation uses single component, but is not limited to do so.
 pub trait LocalPlayer: for<'a> LocalPlayerPack<'a> + 'static {
     /// Query that will be performed during replication process.
-    type Query: Query;
+    type Query: Query + NonTrackingQuery;
 
     /// Replicates state.
     /// Returns serializable data containing all commands for an entity.
@@ -309,16 +314,22 @@ impl LocalPlayer for DummyLocalPlayer {
 /// Allows adding new descriptors and setting local player type.
 pub struct ClientBuilder<I, R> {
     ids: Vec<TypeId>,
-    marker: PhantomData<fn(I, R)>,
+    marker: PhantomData<(I, R)>,
 }
 
 /// Marker type signals that remote player type is not set.
 /// [`ClientBuilder`] starts with this type parameter in place where [`LocalPlayer`] is expected.
 pub enum NoLocalPlayerType {}
 
+impl Default for ClientBuilder<NoLocalPlayerType, ()> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl ClientBuilder<NoLocalPlayerType, ()> {
     /// Returns new un-configured [`ClientBuilder`].
-    pub fn new() -> Self {
+    pub const fn new() -> Self {
         ClientBuilder {
             ids: Vec::new(),
             marker: PhantomData,
@@ -369,24 +380,28 @@ impl<P> ClientBuilder<P, ()> {
     }
 }
 
+type SendInputs = for<'a> fn(
+    &'a HashSet<PlayerId>,
+    &'a mut ClientSession<TcpChannel>,
+    &'a mut World,
+    &'a Scope<'_>,
+) -> Pin<Box<dyn Future<Output = eyre::Result<()>> + 'a>>;
+
+type Replicate = fn(
+    &mut ClientSession<TcpChannel>,
+    &mut EntityMapper,
+    &mut World,
+    scope: &Scope<'_>,
+) -> eyre::Result<()>;
+
 /// Client system that connects to authoritative server.
 /// It should be configured with [`ClientBuilder`] and polled using [`ClientSystem::run`] method.
 pub struct ClientSystem {
     session: Option<ClientSession<TcpChannel>>,
     mapper: EntityMapper,
     controlled: HashSet<PlayerId>,
-    send_inputs: for<'a> fn(
-        &'a HashSet<PlayerId>,
-        &'a mut ClientSession<TcpChannel>,
-        &'a mut World,
-        &'a Scope<'_>,
-    ) -> Pin<Box<dyn Future<Output = eyre::Result<()>> + 'a>>,
-    replicate: fn(
-        &mut ClientSession<TcpChannel>,
-        &mut EntityMapper,
-        &mut World,
-        scope: &Scope<'_>,
-    ) -> eyre::Result<()>,
+    send_inputs: SendInputs,
+    replicate: Replicate,
 }
 
 impl ClientSystem {
@@ -543,7 +558,7 @@ where
     let inputs = scope.to_scope_from_iter(inputs);
     let inputs = inputs
         .iter()
-        .map(|(pid, (nid, input))| ((*pid, (*nid, input))));
+        .map(|(pid, (nid, input))| (*pid, (*nid, input)));
 
     tracing::debug!("Sending input ({})", session.current_step());
     Box::pin(async move {
@@ -643,12 +658,12 @@ macro_rules! for_tuple {
 
                     $(
                         let $b = TrackReplicate::from_mask(mask_idx, &header.mask)?;
-                        replicate_one::<$t>($b, entity, world, &mut cursor)?;
+                        replicate_one::<$t>($b, &entity, world, &mut cursor)?;
                         mask_idx += 1;
                     )+
 
                     let pid = TrackReplicate::from_mask(mask_idx, &header.mask)?;
-                    replicate_one::<PlayerIdDescriptor>(pid, entity, world, &mut cursor)?;
+                    replicate_one::<PlayerIdDescriptor>(pid, &entity, world, &mut cursor)?;
 
                     // End for each component + player_id.
                 }
@@ -660,7 +675,7 @@ macro_rules! for_tuple {
                     })?;
 
                     if let Some(entity) = EntityMapper::get(mapper, nid) {
-                        let _ = world.despawn(entity);
+                        let _ = world.despawn(&entity);
                     }
                 }
 
